///////////////////////////////////////////////////////////////////////////////////
///
///
///	\file	CMessageClient.h
///	\author	Rob Bateman
/// \brief	this file defines the class CMessageClient which is a simple example
///			of a possible use for the underlying templates. 
///	
///

///////////////////////////////////////////////////////////////////////////////////

#pragma once

#ifndef _CMESSAGECLIENT_H_
#define _CMESSAGECLIENT_H_


///////////////////////////////////////////////////////////////////////////////////
//
//	Includes
//
#include "TBaseClient.h"


///////////////////////////////////////////////////////////////////////////////////
///
///	\brief	this class specifies a basic message check to see if the 
///			message is for all people or just you. 
///
struct TMessageCheck
{
private:

	static std::string GetClientName();

public:
	
	///	\brief	the function operator to process the message
	///
	void operator()( std::istream &ifs );
};



	
///	\brief	this is a simple template example of how to build upon the low level 
///			TBaseClient<> template to provide some added functionality.
///	
class CMessageClient : public TBaseClient< TMessageCheck >
{	
public:

	///	\brief	this functions sends a text message to the server
	///	\param	text	-	the message to send
	///	\param	user	-	the user name to whom to send the message (or "all")
	///	\return	true if OK
	///
	static bool SendText( std::string text, std::string user ) ;

};




///	\brief	this template is designed to aid making a command line based server
///
template< typename OnConnection_,typename OnProcessCommand_ >
void RunClientConsole () 
{
	//	instance the function objects
	//
	OnConnection_     OnConnection;
	OnProcessCommand_ OnProcessCommand;
	
	//	buffer for user input
	//
	char buff[BUFFER_SIZE] = {0};
	
	//	temp vars
	//
	std::string server,user;
	
	//	call the function object to specify the user and server name
	//
	OnConnection(server,user);
	
		
	//	keep track of what was last read in
	//
	char* it = buff;
		
	//	Loop 'till the cows come home
	//
	while( 1 )
	{
		//	grab a byte of text
		//
		std::cin.get( *it );
		
		//	if newline,
		//
		if( *it == '\n' )
		{
			std::cout << "> ";
			std::cout.flush();
			
			//	terminate the string		
			//
			*it = 0;
			
			//	process the string
			//
			if(!OnProcessCommand( buff ) )
			{
				return;
			}
			
			//	zero memory and reset ptr
			//
			std::memset(buff,0,512);
			it=buff;
			
		}
		else
		{
			//	move to next char
			//
			++it;
		}
	}
}


#endif